#!/usr/bin/env bash

## "PPHPC MIComp Datasets" by Nuno Fachada are licensed under the 
## Creative Commons Attribution 4.0 International License. To view a 
## copy of this license, visit http://creativecommons.org/licenses/by/4.0/ 
## or send a letter to Creative Commons, PO Box 1866, Mountain View, 
## CA 94042, USA.

# ##################################################### #
# Script used to create the PPHPC MIComp Datasets       #
# All implementations, size 400, param. set 1, 100 runs #
# ##################################################### #

# Location of the Java PPHPC implementation
PPDIR_J="${HOME}/workspace/pphpc/java"

# Location of pphpc.nlogo file (PPHPC NetLogo implementation)
PPDIR_NL="${HOME}/workspace/pphpc/netlogo"

# NetLogo location
NLDIR="${HOME}/LocalPrograms/netlogo-5.2.1"

# Java command to run PPHPC
PPCMD="java -Xms2048m -Xmx4096m -cp ${PPDIR_J}/bin:${PPDIR_J}/lib/* org.laseeb.pphpc.PredPrey"

# Number of threads to use in the Java implementation
THREADS=8

# Location of output files
OUTDIRBASE="./runs100"
OUTDIR_NL="${OUTDIRBASE}/nl_ok"
OUTDIR_JOK="${OUTDIRBASE}/j_ex_ok"
OUTDIR_JNS="${OUTDIRBASE}/j_ex_noshuff"
OUTDIR_JDIFF="${OUTDIRBASE}/j_ex_diff"

# Location of simulation configuration files
CFGDIR="${HOME}/workspace/pphpc/configs"

# Location of alternative simulation configuration files (for Java Diff.)
ALTCFGDIR="alt_configs"

# Model size and parameter set
SIZE=400
VERS=1

# Create output directories if they don't yet exist
mkdir -p ${OUTDIR_NL}
mkdir -p ${OUTDIR_JOK}
mkdir -p ${OUTDIR_JNS}
mkdir -p ${OUTDIR_JDIFF}

# NetLogo OK runs
for RUN in {1..100}
do
    # Determine seed for this run
    SEED=`echo ${RUN} | md5sum | cut -f 1 -d ' '`
    SEED=`echo ${SEED} | awk '{print toupper($0)}'`
    SEED=`echo "ibase=16; ${SEED}" | bc`
    let "SEED=SEED%2147483647"

    let "MULT=2*(SIZE/100)"
    let "INITWOLVES=MULT*SIZE"
    let "INITSHEEP=INITWOLVES*2"
    sed "s/INITSHEEP/${INITSHEEP}/g" nlexps.in.xml > nlexps.xml
    sed -i "s/INITWOLVES/${INITWOLVES}/g" nlexps.xml
    sed -i "s/MAXPXCOR/$(($SIZE-1))/g" nlexps.xml
    sed -i "s/MAXPYCOR/$(($SIZE-1))/g" nlexps.xml
    sed -i "s/RNGSEED/${SEED}/g" nlexps.xml

    OUTFILE="${OUTDIR_NL}/exp${SIZE}v${VERS}r${RUN}.csv"
    if [ ! -f ${OUTFILE} ]; then

        # Print info about current run
        echo "(NL) Run: ${RUN}, Param.Set: ${VERS}, Size: ${SIZE}, SEED: ${SEED}"

        # Perform run
        java -Xms2048m -Xmx4096m -Dfile.encoding=UTF-8 -cp ${NLDIR}/NetLogo.jar org.nlogo.headless.Main --model ${PPDIR_NL}/pphpc.nlogo --setup-file ./nlexps.xml --threads 1 --experiment v${VERS} --table ${OUTFILE}

        # Convert NetLogo 5.2.1 output to TSV (this may fail with other NetLogo versions)
        TSVFILE="${OUTDIR_NL}/stats${SIZE}v${VERS}r${RUN}.txt"
        awk -F, 'NR>7 { gsub("\"", " "); print $19, $20, $21, $22, $23, $24 }' ${OUTFILE} > ${TSVFILE}.tmp

        # Replace spaces with tabs
        sed 's/^ //g' ${TSVFILE}.tmp | sed  's/ \+$//g' | sed 's/ \+/\t/g' > ${TSVFILE}

        # Remove temporary files
        rm ${TSVFILE}.tmp
        rm ${OUTFILE}

    fi

    # Remove temporary configuration file
    rm nlexps.xml

done

# Java OK runs
for RUN in {101..200}
do
    # Determine seed for this run
    SEED=`echo ${RUN} | md5sum | cut -f 1 -d ' '`
    SEED=`echo ${SEED} | awk '{print toupper($0)}'`
    SEED=`echo "ibase=16; ${SEED}" | bc`

    # Equal with reproducibility parallelization strategy (EX)
    OUTFILE="${OUTDIR_JOK}/stats${SIZE}v${VERS}pEXt${THREADS}r${RUN}.txt"
    if [ ! -f ${OUTFILE} ]; then

        # Print info about current run
        echo "(JOK) Run: ${RUN}, Param.Set: ${VERS}, Size: ${SIZE}, SEED: ${SEED}"

        # Perform run
        ${PPCMD} -p ${CFGDIR}/config${SIZE}v${VERS}.txt -s ${OUTFILE} -r ${SEED} -n ${THREADS} -ps EX

    fi
done

# Java NoShuff runs
for RUN in {201..300}
do
    # Determine seed for this run
    SEED=`echo ${RUN} | md5sum | cut -f 1 -d ' '`
    SEED=`echo ${SEED} | awk '{print toupper($0)}'`
    SEED=`echo "ibase=16; ${SEED}" | bc`

    # Equal with reproducibility parallelization strategy (EX)
    OUTFILE="${OUTDIR_JNS}/stats${SIZE}v${VERS}pEXt${THREADS}r${RUN}.txt"
    if [ ! -f ${OUTFILE} ]; then

        # Print info about current run
        echo "(JNS) Run: ${RUN}, Param.Set: ${VERS}, Size: ${SIZE}, SEED: ${SEED}"

        # Perform run
        ${PPCMD} -p ${CFGDIR}/config${SIZE}v${VERS}.txt -s ${OUTFILE} -u -r ${SEED} -n ${THREADS} -ps EX

    fi
done

# Java Diff runs
for RUN in {301..400}
do
    # Determine seed for this run
    SEED=`echo ${RUN} | md5sum | cut -f 1 -d ' '`
    SEED=`echo ${SEED} | awk '{print toupper($0)}'`
    SEED=`echo "ibase=16; ${SEED}" | bc`

    # Equal with reproducibility parallelization strategy (EX)
    OUTFILE="${OUTDIR_JDIFF}/stats${SIZE}v${VERS}pEXt${THREADS}r${RUN}.txt"
    if [ ! -f ${OUTFILE} ]; then

        # Print info about current run
        echo "(JDIFF) Run: ${RUN}, Param.Set: ${VERS}, Size: ${SIZE}, SEED: ${SEED}"

        # Perform run
        ${PPCMD} -p ${ALTCFGDIR}/config${SIZE}v${VERS}.txt -s ${OUTFILE} -r ${SEED} -n ${THREADS} -ps EX

    fi
done
